package net.novagamestudios.kaffeekasse.model.session


sealed interface Session : java.io.Serializable {
    data object Empty : Session
    interface WithDevice : Session {
        val device: Device
    }
    interface WithRealUser : Session {
        val realUser: User
    }
    interface WithDeviceAndUser : Session, WithDevice, WithRealUser

    companion object {
        operator fun invoke(
            device: Device?,
            user: User?
        ): Session {
            val realUser = user?.takeIf { it.isRealUser }
            return when {
                device != null && realUser != null -> SessionWithDeviceAndRealUser(device, realUser)
                device != null -> SessionWithDevice(device)
                realUser != null -> SessionWithRealUser(realUser)
                else -> Empty
            }
        }
    }
}


val Session.deviceOrNull: Device? get() = (this as? Session.WithDevice)?.device
val Session.realUserOrNull: User? get() = (this as? Session.WithRealUser)?.realUser


private val User.isKaffeekasse get() = user == "kaffeekasse"
private val User.isRealUser get() = !isKaffeekasse


private data class SessionWithDevice(
    override val device: Device
) : Session.WithDevice

private data class SessionWithRealUser(
    override val realUser: User
) : Session.WithRealUser

private data class SessionWithDeviceAndRealUser(
    override val device: Device,
    override val realUser: User
) : Session.WithDeviceAndUser
