package net.novagamestudios.kaffeekasse.ui.kaffeekasse

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.expandHorizontally
import androidx.compose.animation.shrinkHorizontally
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.runtime.Composable
import cafe.adriel.voyager.core.model.ScreenModel
import cafe.adriel.voyager.navigator.Navigator
import net.novagamestudios.common_utils.Logger
import net.novagamestudios.kaffeekasse.KaffeekasseModule.Companion.kaffeekasseCartProvider
import net.novagamestudios.kaffeekasse.model.kaffeekasse.MutableCart
import net.novagamestudios.kaffeekasse.model.kaffeekasse.isNotEmpty
import net.novagamestudios.kaffeekasse.model.session.Session
import net.novagamestudios.kaffeekasse.repositories.RepositoryProvider
import net.novagamestudios.kaffeekasse.ui.AppModuleSelection
import net.novagamestudios.kaffeekasse.ui.kaffeekasse.ManualBillScreenModel.Companion.backNavigationHandler
import net.novagamestudios.kaffeekasse.ui.navigation.AppSubpageTitle
import net.novagamestudios.kaffeekasse.ui.navigation.KaffeekasseNavigation
import net.novagamestudios.kaffeekasse.ui.theme.ifAnimationsEnabled
import net.novagamestudios.kaffeekasse.ui.util.navigation.BackNavigationHandler
import net.novagamestudios.kaffeekasse.ui.util.screenmodel.ScreenModelFactory


class KaffeekasseModuleScreenModel private constructor(
    session: Session.WithRealUser,
    val cart: MutableCart
) : ScreenModel, Logger {
    val accountName = when (session) {
        is Session.WithDevice -> session.realUser.displayName ?: "Unknown User"
        else -> null
    }


    companion object : ScreenModelFactory<KaffeekasseNavigation.Tab, KaffeekasseModuleScreenModel> {
        context (RepositoryProvider)
        override fun create(screen: KaffeekasseNavigation.Tab) = KaffeekasseModuleScreenModel(
            session = screen.session,
            cart = kaffeekasseCartProvider[screen.session.realUser]
        )
    }
}




@Composable
fun KaffeekasseTopBarTitle(
    model: KaffeekasseModuleScreenModel,
    navigator: Navigator
) {
    when (navigator.lastItem) {
        is KaffeekasseNavigation.ManualBillScreen -> {
            val name = model.accountName
            if (name != null) AppSubpageTitle("$name")
            else AppModuleSelection()
        }
        is KaffeekasseNavigation.AccountScreen -> AppSubpageTitle("Konto")
        is KaffeekasseNavigation.TransactionsScreen -> AppSubpageTitle("Übersicht")
    }
}

@Composable
fun KaffeekasseTopBarNavigation(
    model: KaffeekasseModuleScreenModel,
    navigator: Navigator
) {
    val backNavigationHandler = model.backNavigationHandler(navigator)
    AnimatedVisibility(
        visible = !backNavigationHandler.canNavigateBack() && model.cart.isNotEmpty(),
        enter = expandHorizontally().ifAnimationsEnabled(),
        exit = shrinkHorizontally().ifAnimationsEnabled()
    ) {
        IconButton(onClick = { model.cart.clear() }) {
            Icon(Icons.Default.Delete, "Empty cart")
        }
    }
}

@Composable
fun KaffeekasseTopBarActions(
    navigator: Navigator
) {
    when (val screen = navigator.lastItem) {
        is KaffeekasseNavigation.ManualBillScreen -> ManualBillTopBarActions(screen.model)
        is KaffeekasseNavigation.AccountScreen -> AccountTopBarActions(screen.model)
        is KaffeekasseNavigation.TransactionsScreen -> TransactionsTopBarActions(screen.model)
    }
}



@Composable
fun KaffeekasseModuleScreenModel.backNavigationHandler(navigator: Navigator): BackNavigationHandler {
    return when (val screen = navigator.lastItem) {
        is KaffeekasseNavigation.ManualBillScreen -> screen.model.backNavigationHandler(navigator)
        is KaffeekasseNavigation.AccountScreen -> BackNavigationHandler.default()
        is KaffeekasseNavigation.TransactionsScreen -> BackNavigationHandler.default()
        else -> BackNavigationHandler.Disabled
    }
}



