package net.novagamestudios.kaffeekasse.ui.navigation

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.expandHorizontally
import androidx.compose.animation.shrinkHorizontally
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.RowScope
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.TopAppBarScrollBehavior
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.unit.dp
import cafe.adriel.voyager.core.screen.Screen
import cafe.adriel.voyager.navigator.Navigator
import net.novagamestudios.kaffeekasse.CrashHandling
import net.novagamestudios.kaffeekasse.ui.theme.LocalAnimationSwitch
import net.novagamestudios.kaffeekasse.ui.theme.ifAnimationsEnabled
import net.novagamestudios.kaffeekasse.ui.util.navigation.BackNavigationHandler
import net.novagamestudios.kaffeekasse.ui.util.navigation.debugNavigation

@Composable
fun AppScaffoldNavigator(
    key: String,
    initialRoute: List<Screen>,
    title: @Composable (Navigator) -> Unit = { },
    navigationIcon: @Composable (Navigator) -> Unit = { },
    actions: @Composable RowScope.(Navigator) -> Unit = { },
    topAppBarScrollBehavior: TopAppBarScrollBehavior? = TopAppBarDefaults.enterAlwaysScrollBehavior(),
    backNavigationHandlerProvider: @Composable (Navigator) -> BackNavigationHandler = { BackNavigationHandler.default() },
    content: @Composable Navigator.() -> Unit = { AppScaffoldNavigatorDefaultContent("$key-content") }
) = Navigator(
    screens = initialRoute,
    onBackPressed = null,
    key = key
) { navigator ->
    debugNavigation()
    CrashHandling.updateNavigation(navigator)
    val backNavigationHandler = backNavigationHandlerProvider(navigator)
    AppScaffold(
        topBar = {
            AppTopBar(
                title = { title(navigator) },
                navigationIcon = {
                    DefaultBackNavigation(backNavigationHandler)
                    navigationIcon(navigator)
                },
                actions = { actions(navigator) },
                scrollBehavior = topAppBarScrollBehavior
            )
        },
        topAppBarScrollBehavior = topAppBarScrollBehavior
    ) {
        navigator.content()
        backNavigationHandler.BackHandler()
    }
}

@Composable
fun Navigator.AppScaffoldNavigatorDefaultContent(
    key: String
) {
    AppScreenTransition(this, key = key) { screen ->
        screen.Content()
    }
}

@Composable
private fun AppScaffold(
    modifier: Modifier = Modifier,
    topBar: @Composable () -> Unit = { },
    topAppBarScrollBehavior: TopAppBarScrollBehavior? = null,
    content: @Composable () -> Unit
) = Scaffold(
    modifier.run {
        if (topAppBarScrollBehavior == null || !LocalAnimationSwitch.current) this
        else nestedScroll(topAppBarScrollBehavior.nestedScrollConnection)
    },
    topBar = topBar
) { paddingValues ->
    Box(
        Modifier
            .padding(paddingValues)
            .fillMaxSize(),
        propagateMinConstraints = true
    ) {
        content()
    }
}

@Composable
fun AppTopBar(
    title: @Composable () -> Unit,
    modifier: Modifier = Modifier,
    navigationIcon: @Composable () -> Unit = {},
    actions: @Composable RowScope.() -> Unit = {},
    scrollBehavior: TopAppBarScrollBehavior? = null
) = TopAppBar(
    title = { title() },
    modifier = modifier,
    navigationIcon = { navigationIcon() },
    actions = { actions() },
    colors = TopAppBarDefaults.topAppBarColors(
        scrolledContainerColor = MaterialTheme.colorScheme.surface
    ),
    scrollBehavior = scrollBehavior.takeIf { LocalAnimationSwitch.current }
)

@Composable
fun AppSubpageTitle(text: String) = Text(text, Modifier.padding(start = 8.dp))

@Composable
fun DefaultBackNavigation(
    handler: BackNavigationHandler
) {
    AnimatedVisibility(
        visible = handler.canNavigateBack(),
        enter = expandHorizontally().ifAnimationsEnabled(),
        exit = shrinkHorizontally().ifAnimationsEnabled()
    ) {
        IconButton(onClick = { handler.onNavigateBack() }) {
            Icon(Icons.AutoMirrored.Filled.ArrowBack, "Back")
        }
    }
}