package net.novagamestudios.kaffeekasse.ui.util

import androidx.compose.foundation.layout.BoxScope
import androidx.compose.material3.pulltorefresh.PullToRefreshState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import kotlinx.coroutines.launch
import net.novagamestudios.common_utils.compose.components.Progress
import net.novagamestudios.kaffeekasse.ui.kaffeekasse.components.FailureRetryScreen
import net.novagamestudios.kaffeekasse.util.richdata.RichData
import net.novagamestudios.kaffeekasse.util.richdata.RichDataFunctions
import net.novagamestudios.kaffeekasse.util.richdata.RichDataSource
import net.novagamestudios.kaffeekasse.util.richdata.RichDataStateWithFunctions
import net.novagamestudios.kaffeekasse.util.richdata.collectAsRichState


@Composable
fun <T : Any> PullToRefreshBox(
    source: RichDataSource<T>,
    modifier: Modifier = Modifier,
    content: @Composable BoxScope.() -> Unit
) = PullToRefreshBox(
    state = source.collectAsRichState(),
    modifier = modifier,
    content = content
)

@Composable
fun <T : Any> PullToRefreshBox(
    state: RichDataStateWithFunctions<T>,
    modifier: Modifier = Modifier,
    content: @Composable BoxScope.() -> Unit
) {
    val coroutineScope = rememberCoroutineScope()
    PullToRefreshBox(
        refreshing = { state.isLoading },
        onRefresh = { coroutineScope.launch { state.refresh() } },
        modifier,
        content = content
    )
}

@Composable
fun <T : Any> rememberPullToRefreshState(
    data: RichDataSource<T>,
): PullToRefreshState {
    val dataState = data.collectAsRichState()
    val coroutineScope = rememberCoroutineScope()
    return rememberPullToRefreshState(
        refreshing = { dataState.isLoading },
        onRefresh = { coroutineScope.launch { data.refresh() } }
    )
}

@Composable
inline fun <T : Any> RichDataContent(
    source: RichDataSource<T>,
    ensureClean: Boolean = true,
    errorContent: @Composable RichDataContentScope<T>.(RichData.Error<T>) -> Unit = { },
    loadingContent: @Composable RichDataContentScope<T>.(Progress) -> Unit = { },
    noneContent: @Composable RichDataContentScope<T>.() -> Unit = { },
    dataContent: @Composable RichDataContentScope<T>.(T) -> Unit
) = RichDataContent(
    state = source.collectAsRichState(),
    ensureClean = ensureClean,
    errorContent = errorContent,
    loadingContent = loadingContent,
    noneContent = noneContent,
    dataContent = dataContent
)


@Composable
inline fun <T : Any> RichDataContent(
    state: RichDataStateWithFunctions<T>,
    ensureClean: Boolean = true,
    errorContent: @Composable RichDataContentScope<T>.(RichData.Error<T>) -> Unit = { },
    loadingContent: @Composable RichDataContentScope<T>.(Progress) -> Unit = { },
    noneContent: @Composable RichDataContentScope<T>.() -> Unit = { },
    dataContent: @Composable RichDataContentScope<T>.(T) -> Unit
) {
    if (ensureClean) LaunchedEffect(Unit) { state.ensureCleanData() }
    val scope = remember(state) { object : RichDataContentScope<T> { override val functions = state } }
    when (val value = state.value) {
        is RichData.None<T> -> scope.noneContent()
        is RichData.Loading<T> -> scope.loadingContent(value.progress)
        is RichData.Data<T> -> scope.dataContent(value.data)
        is RichData.Error<T> -> scope.errorContent(value)
    }
}

interface RichDataContentScope<T : Any> {
    val functions: RichDataFunctions<T>
}


context (RichDataContentScope<T>)
@Composable
fun <T : Any> FailureRetryScreen(
    error: RichData.Error<T>,
    message: String,
    modifier: Modifier = Modifier
) {
    val coroutineScope = rememberCoroutineScope()
    FailureRetryScreen(
        message = message,
        errors = error.messages,
        onRetry = { coroutineScope.launch { functions.refresh() } },
        modifier = modifier
    )
}


