package net.novagamestudios.kaffeekasse.ui.util.navigation

import androidx.compose.runtime.Composable
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import cafe.adriel.voyager.navigator.LocalNavigator
import cafe.adriel.voyager.navigator.Navigator
import cafe.adriel.voyager.navigator.currentOrThrow

interface BackNavigationHandler {

    /**
     * @return true if back navigation ui components should be shown.
     * false does not mean that the navigation can not be handled
     */
    fun canNavigateBack(): Boolean

    /**
     * @return true if the navigation was handled and false if it should be passed on
     */
    fun onNavigateBack(): Boolean

    @Composable
    fun BackHandler() = androidx.activity.compose.BackHandler {
        onNavigateBack()
    }

    companion object {
        val Disabled = object : BackNavigationHandler {
            override fun canNavigateBack() = false
            override fun onNavigateBack() = true
            @Composable
            override fun BackHandler() { }
        }
        fun forNavigator(navigator: Navigator) = object : BackNavigationHandler {
            override fun canNavigateBack() = navigator.canPop
            override fun onNavigateBack(): Boolean {
                navigator.pop()
                return true
            }
        }
        @Composable
        fun default(): BackNavigationHandler {
            val navigator = LocalNavigator.currentOrThrow
            return remember(navigator) { forNavigator(navigator) }
        }
        infix fun BackNavigationHandler?.then(parent: BackNavigationHandler?) = object :
            BackNavigationHandler {
            override fun canNavigateBack(): Boolean {
                return this@then?.canNavigateBack() == true || parent?.canNavigateBack() == true
            }
            override fun onNavigateBack(): Boolean {
                return this@then?.onNavigateBack() == true || parent?.onNavigateBack() == true
            }
        }

        fun derivedOf(
            canNavigateBack: () -> Boolean,
            onNavigateBack: () -> Boolean
        ) = object : BackNavigationHandler {
            private val _canNavigateBack by derivedStateOf(canNavigateBack)
            override fun canNavigateBack() = _canNavigateBack
            override fun onNavigateBack() = onNavigateBack()
        }
    }
}
