package net.novagamestudios.kaffeekasse.util

import kotlinx.coroutines.sync.Mutex
import kotlinx.coroutines.sync.withLock
import kotlinx.coroutines.withContext
import kotlin.coroutines.CoroutineContext
import kotlin.coroutines.coroutineContext


//class ReentrantMutex {
//    private val mutex = Mutex()
//    val isLocked get() = mutex.isLocked
//    suspend fun <T> withReentrantLock(block: suspend () -> T) = mutex.withReentrantLock(block)
//}

suspend fun <T> Mutex.withReentrantLock(block: suspend () -> T): T {
    val key = ReentrantMutexContextKey(this)
    if (coroutineContext[key] != null) return block()
    return withContext(ReentrantMutexContextElement(key)) { withLock { block() } }
}

private class ReentrantMutexContextElement(override val key: ReentrantMutexContextKey) : CoroutineContext.Element
private data class ReentrantMutexContextKey(val mutex: Mutex) : CoroutineContext.Key<ReentrantMutexContextElement>
