package net.novagamestudios.kaffeekasse.util

import kotlinx.coroutines.flow.FlowCollector
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.sync.Mutex
import kotlinx.datetime.LocalTime
import kotlin.time.Duration
import kotlin.time.Duration.Companion.nanoseconds


operator fun LocalTime.minus(other: LocalTime): Duration {
    return (toNanosecondOfDay() - other.toNanosecondOfDay()).nanoseconds
}


inline fun Mutex.tryWithLock(owner: Any? = null, block: () -> Unit): Boolean {
    if (!tryLock(owner)) return false
    try {
        block()
    } finally {
        unlock(owner)
    }
    return true
}

fun <T, R> StateFlow<T>.mapState(
    transform: (T) -> R
): StateFlow<R> = object : StateFlow<R> {
    override val replayCache: List<R> get() = this@mapState.replayCache.map(transform)
    override val value: R get() = transform(this@mapState.value)
    override suspend fun collect(collector: FlowCollector<R>): Nothing {
        this@mapState.collect { value -> collector.emit(transform(value)) }
    }
}

context (T)
fun <T> context(): T = this@T
